component {

    property name="systemConfigurationService" inject="delayedInjector:systemConfigurationService";
    property name="logger" inject="logbox:logger:restsecurity";

    public void function configure() {}

    public void function postPresideReload( event, interceptData ) {
        // check if system settings are defined, otherwise set default values
        _ensureSystemSettings();
    }
    
    public void function onRestRequest( event, interceptData ) {

        var canError        = logger.canError();
        var canWarn         = logger.canWarn();
        var canInfo         = logger.canInfo();

        var restRequest     = arguments.interceptData.restRequest;
        var restResponse    = arguments.interceptData.restResponse;
        var validAPIKey     = _getValidAPIKey();

        //writeDump(restRequest);abort;

        // TODO: maybe support dev environment (in dev env automatically set valid api key)
        // e.g. use a flag in system settings to bypass the check if working locally

        if (len(validAPIKey) eq 0) {
            // no api key defined in system settings, rest api cannot be used
            restResponse.noData()
                .setStatus( 500, "Server Error" )
                .setHeader( "X-ERROR-MESSAGE",  "REST API not ready. No API Key defined in system settings." );
            restRequest.finish();
            if ( canWarn ) {
                logger.warn( "REST API not ready. No API Key defined in system settings." );
            }
            return;
        }

        var httpHeaderName = _getAPIKeyHTTPHeaderName();
        var httpRequestData = getHTTPRequestData();
        var headers = httpRequestData.headers;

        if (!headers.keyExists(httpHeaderName)) {
            if ( canError ) {
                logger.error( "#restRequest.getVerb()# #restRequest.getURI()#: 401, not authenticated, Missing HTTP header '#httpHeaderName#'" );
                //logger.error( "401, not authenticated, Missing HTTP header '#httpHeaderName#'" );
            }
            restResponse.noData()
                .setStatus( 401, "Not Authenticated" )
                .setHeader( "X-ERROR-MESSAGE",  "Missing HTTP header '#httpHeaderName#'" );
            restRequest.finish();
            return;
        }

        if (headers[httpHeaderName] neq validAPIKey) {
            if ( canError ) {
                logger.error( "#restRequest.getVerb()# #restRequest.getURI()#: 403, not authorized, Invalid API Key '#headers[httpHeaderName]#' in HTTP header '#httpHeaderName#'" );
                //logger.error( "403, not authorized, Invalid API Key '#headers[httpHeaderName]#' in HTTP header '#httpHeaderName#'" );
            }
            restResponse.noData()
                .setStatus( 403, "Not Authorized" )
                .setHeader( "X-ERROR-MESSAGE",  "Invalid API Key. Please use a valid API key in HTTP header '#httpHeaderName#'" );
            restRequest.finish();
            return;
        }
    }

// PRIVATE HELPERS
    private string function _getValidAPIKey() {
        return systemConfigurationService.getSetting(
              category = "rest-security"
            , setting  = "api_key"
            , default  = ""
        );
    }

    private string function _getAPIKeyHTTPHeaderName(string defaultValue="X-API-KEY") {
        return systemConfigurationService.getSetting(
              category = "rest-security"
            , setting  = "http_request_header_name"
            , default  = arguments.defaultValue
        );
    }

    private void function _ensureSystemSettings() {
        
        var canInfo         = logger.canInfo();
        var apikey          = _getValidAPIKey();
        var httpHeaderName  = _getAPIKeyHTTPHeaderName(defaultValue="");

        if (len(apikey) eq 0) {
            systemConfigurationService.saveSetting(
                  category = "rest-security"
                , setting  = "api_key"
                , value  = hash(createUUID())
            );
            if ( canInfo ) {
                logger.info( "No API Key defined, generated and saved default value." );
            }
        }
        if (len(httpHeaderName) eq 0) {
            systemConfigurationService.saveSetting(
                  category = "rest-security"
                , setting  = "http_request_header_name"
                , value  = "X-API-KEY"
            );
            if ( canInfo ) {
                logger.info( "No HTTP request header name defined, saved default 'X-API-KEY'." );
            }
        }
    }
}